<?php

declare(strict_types=1);

namespace Dedi\SyliusContactPlugin\Entity;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Sylius\Component\Core\Model\AdminUserInterface;

/**
 * @ORM\Entity()
 * @ORM\Table(name="dedi_sylius_contact_answer")
 */
class Answer implements AnswerInterface
{
    /**
     * @ORM\Column(name="id", type="integer", nullable=false)
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected ?int $id;

    /**
     * @ORM\ManyToOne(targetEntity="Dedi\SyliusContactPlugin\Entity\ContactFormMessage", inversedBy="answers")
     * @ORM\JoinColumn(name="contact_form_message_id", referencedColumnName="id")
     */
    protected ?ContactFormMessageInterface $contactFormMessage;

    /**
     * @ORM\ManyToOne(targetEntity="Sylius\Component\Core\Model\AdminUser")
     * @ORM\JoinColumn(name="author_id", referencedColumnName="id")
     */
    protected ?AdminUserInterface $author;

    /**
     * @ORM\OneToMany(targetEntity="Dedi\SyliusContactPlugin\Entity\Attachment", mappedBy="answer", cascade={"persist"})
     */
    private Collection $attachments;

    /**
     * @ORM\Column(name="message", type="text", nullable=false)
     */
    protected ?string $message;

    /**
     * @ORM\Column(name="created_at", type="datetime", nullable=false)
     */
    protected ?\DateTime $createdAt;

    public function __construct()
    {
        $this->attachments = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getAuthor(): ?AdminUserInterface
    {
        return $this->author;
    }

    public function setAuthor(?AdminUserInterface $author): AnswerInterface
    {
        $this->author = $author;

        return $this;
    }

    public function getMessage(): ?string
    {
        return $this->message;
    }

    public function setMessage(?string $message): AnswerInterface
    {
        $this->message = $message;

        return $this;
    }

    public function getContactFormMessage(): ContactFormMessageInterface
    {
        return $this->contactFormMessage;
    }

    public function setContactFormMessage(?ContactFormMessageInterface $contactFormMessage): AnswerInterface
    {
        $this->contactFormMessage = $contactFormMessage;

        return $this;
    }

    public function getCreatedAt(): \DateTime
    {
        return $this->createdAt;
    }

    public function setCreatedAt(\DateTime $createdAt): AnswerInterface
    {
        $this->createdAt = $createdAt;

        return $this;
    }

    public function getAttachments(): Collection
    {
        return $this->attachments;
    }

    public function addAttachment(AttachmentInterface $attachment): self
    {
        if (!$this->hasAttachment($attachment)) {
            $attachment->setAnswer($this);
            $this->attachments->add($attachment);
        }

        return $this;
    }

    public function removeAttachment(AttachmentInterface $attachment): self
    {
        if ($this->hasAttachment($attachment)) {
            $attachment->setAnswer(null);
            $this->attachments->removeElement($attachment);
        }

        return $this;
    }

    public function hasAttachment(AttachmentInterface $attachment): bool
    {
        return $this->attachments->contains($attachment);
    }
}
