<?php

declare(strict_types=1);

namespace Dedi\SyliusContactPlugin\Entity;

use DateTime;
use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Doctrine\ORM\Mapping as ORM;
use Gedmo\Timestampable\Traits\TimestampableEntity;

/**
 * Class ContactFormMessage
 *
 * @ORM\Entity()
 * @ORM\Table(name="dedi_sylius_contact_message")
 */
class ContactFormMessage implements ContactFormMessageInterface
{
    use TimestampableEntity;

    /**
     * @ORM\Column(name="id", type="integer", nullable=false)
     * @ORM\Id()
     * @ORM\GeneratedValue(strategy="AUTO")
     */
    protected ?int $id;

    /**
     * @ORM\ManyToOne(targetEntity="Dedi\SyliusContactPlugin\Entity\ContactFormSubjectInterface")
     * @ORM\JoinColumn(name="subject_id", referencedColumnName="id")
     */
    protected ContactFormSubjectInterface $subject;

    /**
     * @ORM\Column(name="name", type="string")
     */
    protected ?string $name;

    /**
     * @ORM\Column(name="email", type="string", nullable=false)
     */
    protected ?string $email;

    /**
     * @ORM\Column(name="phone", type="string", nullable=true)
     */
    protected ?string $phone;

    /**
     * @ORM\Column(name="message", type="text", nullable=false)
     */
    protected ?string $message;

    /**
     * @ORM\Column(name="ip", type="string")
     */
    protected ?string $ip;

    /**
     * @ORM\Column(name="process_state", type="string")
     */
    protected ?string $processState;

    /**
     * @ORM\OneToMany(targetEntity="Dedi\SyliusContactPlugin\Entity\Answer", mappedBy="contactFormMessage", cascade={"persist"})
     */
    private Collection $answers;

    /**
     * @ORM\OneToMany(targetEntity="Dedi\SyliusContactPlugin\Entity\Attachment", mappedBy="contactFormMessage", cascade={"persist"})
     */
    private Collection $attachments;

    public function __construct()
    {
        $this->answers = new ArrayCollection();
        $this->attachments = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getSubject(): ContactFormSubjectInterface
    {
        return $this->subject;
    }

    public function setSubject(ContactFormSubjectInterface $subject): self
    {
        $this->subject = $subject;

        return $this;
    }

    public function getName(): ?string
    {
        return $this->name;
    }

    public function setName(?string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function getPhone(): ?string
    {
        return $this->phone;
    }

    public function setPhone(?string $phone): self
    {
        $this->phone = $phone;

        return $this;
    }

    public function getMessage(): ?string
    {
        return $this->message;
    }

    public function setMessage(?string $message): self
    {
        $this->message = $message;

        return $this;
    }

    public function getIp(): ?string
    {
        return $this->ip;
    }

    public function setIp(?string $ip): self
    {
        $this->ip = $ip;

        return $this;
    }

    public function getProcessState(): ?string
    {
        return $this->processState;
    }

    public function setProcessState(?string $processState): self
    {
        $this->processState = $processState;

        return $this;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(?string $email): self
    {
        $this->email = $email;

        return $this;
    }

    public function getCreatedAt(): DateTime
    {
        return $this->createdAt;
    }

    public function getUpdatedAt(): DateTime
    {
        return $this->updatedAt;
    }

    public function getAnswers(): Collection
    {
        return $this->answers;
    }

    public function addAnswer(AnswerInterface $answer): self
    {
        if (!$this->hasAnswer($answer)) {
            $answer->setContactFormMessage($this);
            $this->answers->add($answer);
        }

        return $this;
    }

    public function removeAnswer(AnswerInterface $answer): self
    {
        if ($this->hasAnswer($answer)) {
            $answer->setContactFormMessage(null);
            $this->answers->removeElement($answer);
        }

        return $this;
    }

    public function hasAnswer(AnswerInterface $answer): bool
    {
        return $this->answers->contains($answer);
    }

    public function getAttachments(): Collection
    {
        return $this->attachments;
    }

    public function addAttachment(AttachmentInterface $attachment): self
    {
        if (!$this->hasAttachment($attachment)) {
            $attachment->setContactFormMessage($this);
            $this->attachments->add($attachment);
        }

        return $this;
    }

    public function removeAttachment(AttachmentInterface $attachment): self
    {
        if ($this->hasAttachment($attachment)) {
            $attachment->setContactFormMessage(null);
            $this->attachments->removeElement($attachment);
        }

        return $this;
    }

    public function hasAttachment(AttachmentInterface $attachment): bool
    {
        return $this->attachments->contains($attachment);
    }
}
