<?php

declare(strict_types=1);

namespace Dedi\SyliusContactPlugin\Entity;

use Doctrine\ORM\Mapping as ORM;
use Sylius\Component\Resource\Model\TranslatableTrait;
use Sylius\Component\Resource\Model\TranslationInterface;

/**
 * Class ContactFormSubject
 *
 * @ORM\Entity()
 * @ORM\Table(name="dedi_sylius_contact_subject")
 */
class ContactFormSubject implements ContactFormSubjectInterface
{
    use TranslatableTrait {
        __construct as private initializeTranslationsCollection;
    }

    public function __construct()
    {
        $this->initializeTranslationsCollection();
    }

    /**
     * @ORM\Column(name="id", type="integer", nullable=false)
     * @ORM\GeneratedValue(strategy="AUTO")
     * @ORM\Id()
     */
    protected ?int $id;

    /**
     * @ORM\Column(name="active", type="boolean")
     */
    protected bool $active = true;

    /**
     * @ORM\Column(name="subject_rank", type="smallint")
     */
    protected ?int $subjectRank = 10;

    public function getId(): ?int
    {
        return $this->id;
    }

    public function isActive(): bool
    {
        return $this->active;
    }

    public function setActive(bool $active): self
    {
        $this->active = $active;

        return $this;
    }

    public function getSubjectRank(): ?int
    {
        return $this->subjectRank;
    }

    public function setSubjectRank(?int $subjectRank): self
    {
        $this->subjectRank = $subjectRank;

        return $this;
    }

    protected function createTranslation(): TranslationInterface
    {
        return new ContactFormSubjectTranslation();
    }

    public function getLabel(): string
    {
        /** @var ContactFormSubjectTranslation $translation */
        $translation = $this->getTranslation();

        return $translation->getLabel();
    }

    public function getRecipientEmail(): string
    {
        /** @var ContactFormSubjectTranslation $translation */
        $translation = $this->getTranslation();

        return $translation->getRecipientEmail();
    }

    public function __toString()
    {
        return $this->getLabel();
    }
}
